const promise = require('bluebird')
const mysql = require("mysql");

const config = require('./config')

let connection

class DB {
    async getConnection() {
        return new promise((resolve, reject) => {
            connection = mysql.createConnection({
                host: config.db.host,
                user: config.db.user,
                password: config.db.password,
                database: config.db.database,
            });

            connection.connect(async (err) => {
                if (err) {
                    // console.error('error connecting: ' + err.stack)
                    reject('Error while connectiong database !')
                }
                // console.log(`connected to ${config.db.database} as id ${connection.processID}`)
                resolve('Database Connected !')
            })

            connection.on('error', async (err) => {
                // console.log('============db error==============', err);
                // if (err.code === 'PROTOCOL_CONNECTION_LOST') { // Connection to the MySQL server is usually
                await this.getConnection();
                // } else {
                throw err;
                // }
            });
        })
    }

    select(table, selectParams, condition) {
        return new promise((resolve, reject) => {
            let query = `SELECT ${selectParams} FROM ${table}`
            if (condition) {
                query += ` WHERE ${condition}`
            }

            connection.query(query, (error, results) => {
                if (error) {
                    console.log(error)
                    reject('DB_ERROR')
                } else {
                    resolve(results)
                }
            })
        })
    }

    getById(table, selectParams, condition) {
        return new promise((resolve, reject) => {
            let query = `SELECT ${selectParams} FROM ${table}`
            if (condition) {
                query += ` WHERE ${condition}`
            }

            connection.query(query, (error, results) => {
                if (error) {
                    reject('DB_ERROR')
                } else {
                    if (results.length > 0) {
                        resolve(results[0]);
                    } else {
                        resolve(results);
                    }
                }
            })
        })
    }

    get_join(data) {
        return new promise((resolve, reject) => {

            let limit = data.limit ? data.limit : '';
            let where = data.where ? data.where : '';
            let table = data.table ? data.table : '';
            let group_by = data.group_by ? data.group_by : '';
            let order_by_field = data.order_by_field ? data.order_by_field : '';
            let order_by = data.order_by ? data.order_by : '';
            let selectParams = data.selectParams ? data.selectParams : '';
            let joins = data.joins ? data.joins : '';

            let query = `SELECT ${selectParams} FROM ${table}`

            if (joins) {
                for (const key in joins) {
                    query += ` LEFT JOIN ${joins[key]['table']} ON ${joins[key]['join']}`;
                }
            }

            if (where) {
                query += ` WHERE ${where}`
            }

            if (order_by_field) {
                query += ` order by ${order_by_field} ${order_by}`
            }

            if (group_by) {
                query += ` GROUP BY (${group_by})`
            }

            if (limit) {
                query += ` limit ${limit}`
            }

            console.log(query);
            connection.query(query, (error, results) => {
                if (error) {
                    reject('DB_ERROR')
                } else {
                    resolve(results);
                }
            })
        })
    }

    insert(table, data) {
        return new promise((resolve, reject) => {
            let query = `INSERT INTO ${table}(${Object.keys(data).join(',')})  VALUES(${Object.values(data).map(d => ("'" + d + "'"))})`
            console.log("query", query)
            connection.query(query, (error, results) => {
                if (error) {
                    console.log(error)
                    reject('DB_ERROR')
                } else {
                    resolve(results.insertId)
                }
            })
        })
    }

    update(table, condition, data) {
        return new promise((resolve, reject) => {
            let query = `UPDATE ${table} SET ${Object.entries(data).map(entry =>
                (entry[0] + '=' + ((entry[1] == null ? entry[1] : "'" + String(entry[1]).replace(/'/g, "''") + "'"))))} WHERE ${condition}`
            // console.log("query==============", query)
            connection.query(query, (error, results) => {
                if (error) {
                    console.log("error", error)
                    reject('DB_ERROR')
                } else {
                    // console.log("results=========",results)
                    resolve(results)
                }
            })
        })
    }

    upsert(table, data, conflict_key) {
        return new promise((resolve, reject) => {
            let query = `INSERT INTO ${table}(${Object.keys(data).join(',')}) VALUES( 
                  ${Object.values(data).map(d => ("'" + d + "'"))}) ON CONFLICT 
                  (${conflict_key}) DO UPDATE SET 
                  ${Object.entries(data).map(entry => (entry[0] + '=' + "'" + entry[1] + "'"))} RETURNING   ${conflict_key}`

            connection.query(query, (error, results) => {
                if (error) {
                    console.log(error)
                    reject('DB_ERROR')
                } else {
                    resolve(results)
                }
            })
        })
    }

    delete(table, condition) {
        return new promise((resolve, reject) => {
            let query = `DELETE FROM ${table} WHERE ${condition}`
            connection.query(query, (error, results) => {
                if (error) {
                    console.log(error)
                    reject('DB_ERROR')
                } else {
                    resolve(results)
                }
            })
        })
    }

    custom(query) {
        return new promise((resolve, reject) => {
            // console.log("\n Custom query ================ ", query);
            connection.query(query, (error, results) => {
                if (error) {
                    console.log(error)
                    reject('DB_ERROR')
                } else {
                    resolve(results)
                }
            })
        })
    }

    bulkinsert(table, fields, values, condition, updvalues) {
        return new promise((resolve, reject) => {
            let query = ''
            if (condition != '') {
                query = `INSERT INTO ${table} ${fields} VALUES ${values} ON CONFLICT (${condition}) DO UPDATE SET  ${updvalues} `
            } else {
                query = `INSERT INTO ${table} ${fields} VALUES ${values} ON CONFLICT DO NOTHING `
            }
            // console.log("query", query)
            connection.query(query, (error, results) => {
                if (error) {
                    console.log(error)
                    reject('DB_ERROR')
                } else {
                    resolve(results.rows[0])
                }
            })
        })
    }
}


module.exports = new DB()
