const db = require('../utils/db')
const config = require('../utils/config')
const dateHelper = require('../utils/dateHelper')
const messages = require('../utils/messages.json')
const FCM = require('fcm-node')
let FCM_SERVER_KEY
let fcm

class NotificationHelper {
  insertNotification(notify_id, title, text, sender_id, receiver_id, notify_type, unique_id, language, notification_table, is_from_customer, date) {
    let data = {
      notify_id: notify_id,
      title: messages[language][title],
      text: messages[language][text],
      sender_id: sender_id,
      receiver_id: receiver_id,
      notify_type: notify_type,
      unique_id: unique_id,
      is_read: 0,
      created_date: dateHelper.getCurrentTimeStamp(),
      modified_date: dateHelper.getCurrentTimeStamp(),
      is_deleted: 0,
      is_from_customer: is_from_customer
    }
    if (notification_table == 'service_provider_notifications' && notify_type == 6) {
      data.text = messages[language][text].replace('@@DATE@@', date)
    }
    // else if (notify_type == 1 || notify_type == 3 || notify_type == 5 || notify_type == 9 || notify_type == 4 || notify_type == 10) {
    //   data.text = messages[language][text].replace('@@USER_NAME@@', user_name).replace('@@DATE@@', moment(order_request_date).format('LL')).replace('@@TIME@@', moment(order_request_time, 'hh:mm:ss a').format('hh:mm A'))
    // } 
    db.insert(notification_table, data)
    return data
  }

  async insertAndSendNotificationAdmin(notify_id, title, text, notification_is_for, notify_type, unique_id, language, notification_table) {
    const notification_role = {
      REQUEST_FOR_LEAVE: "Manage Leaves",
      CANCEL_SUBSCRIPTION_PLAN_REQUEST: "Cancel Subscription",
      REQUEST_FOR_DEMO: "Demo Request"
    }
    let selectParams = ` admin.admin_id, JSON_AGG(adr.device_token) AS device_tokens`,
      joins = ` LEFT JOIN admin_device_realation adr ON adr.admin_id=admin.admin_id
              LEFT JOIN roles ON roles.role_id=admin.role_id `,
      condition = ` admin.role_id=0 OR roles.roles_json->'${notification_role[notification_is_for]}' IS NOT NULL `,
      pagination = ` GROUP BY admin.admin_id `
    const admins = await db.select('admin' + joins, selectParams, condition + pagination),
      receiver_id = []
    let device_tokens = []
    admins.forEach(token => {
      const data = {
        notify_id: notify_id,
        title: title,
        text: text,
        receiver_id: token.admin_id,
        notify_type: notify_type,
        unique_id: unique_id,
        is_read: 0,
        created_date: dateHelper.getCurrentTimeStamp(),
        modified_date: dateHelper.getCurrentTimeStamp(),
      }
      db.insert(notification_table, data)
      if (token.device_tokens[0]) {
        device_tokens = [...device_tokens, ...token.device_tokens]
      }
      this.sendNotificationAdmin(data, device_tokens)
    })
    return true
  }

  async sendNotification(data, notification_table, notification_id, device_table, user_id_column_name, user_type) {
    let unread_notification_count = 0, unread_chat_message_count = 0,
      unread_notifications = await db.select(notification_table, ` COUNT(${notification_id}) as unread_count`, ` receiver_id = ${data.receiver_id} AND is_read = 0`)
    unread_notification_count = Number(unread_notifications[0].unread_count)
    let chat_user_id = await db.select('chat_user', `chat_user_id`, ` user_type =${user_type} AND user_id = ${data.receiver_id}`)
    if (chat_user_id.length > 0) {
      let unread_chat_messages = await db.select('chat', ` COUNT(chat) as unread_count`, ` receiver_id = ${chat_user_id[0].chat_user_id} AND is_read = 0`)
      unread_chat_message_count = Number(unread_chat_messages[0].unread_count)
    }
    let count = unread_notification_count + unread_chat_message_count  // badge count
    let device_data = await db.select(device_table, `device_token, device_type`, ` ${user_id_column_name} = ${data.receiver_id} AND allow_notification = 1`)
    let android_device_token = [], ios_device_token = []
    for (let d of device_data) {
      if (d.device_type == 1) {
        android_device_token.push(d.device_token)
      } else {
        ios_device_token.push(d.device_token)
      }
    }
    if (notification_table == 'customer_notifications') {
      FCM_SERVER_KEY = config.FCMApiKeyTekoto
    } else if (notification_table == 'service_provider_notifications') {
      FCM_SERVER_KEY = config.FCMApiKeyTekotoJobs
    }
    if (android_device_token.length > 0) {
      this.buildFCM(data.title, data.text, data, count, 1, android_device_token, FCM_SERVER_KEY)
    }
    if (ios_device_token.length > 0) {
      this.buildFCM(data.title, data.text, data, count, 0, ios_device_token, FCM_SERVER_KEY)
    }
  }

  async sendNotificationAdmin(data, device_tokens) {
    let admin_device_token = device_tokens
    if (!device_tokens) {
      let device_data = await db.select('admin_device_realation', `device_token`, ` admin_id = ${data.receiver_id} `)
      for (let d of device_data) {
        admin_device_token.push(d.device_token)
      }
    }
    FCM_SERVER_KEY = config.FCMApiKeyTekotoJobs
    if (admin_device_token.length > 0) {
      this.buildFCMAdmin(data, admin_device_token, FCM_SERVER_KEY)
    }
  }

  buildFCM(title, body, data, count, device_type, device_token, FCM_SERVER_KEY) {
    let payload = {}
    payload.priority = 'high'
    payload.registration_ids = device_token

    data.sound = 'default'
    data.badge = count
    if (device_type == 0) {
      payload.notification = {}
      payload.notification.title = title
      payload.notification.body = body
      for (let key in data) {
        payload.notification[key] = data[key]
      }
      payload.notification.badge = data.badge
    } else if (device_type == 1) {
      payload.data = data
      payload.data.priority = 'high'
      payload.data.show_in_foreground = 'true'
      payload.notification = {}
      payload.notification = data
      payload.notification.priority = 'high'
      payload.notification.show_in_foreground = 'true'
    }
    console.log("=======Payload=======", payload)
    this.sendFCM(payload, FCM_SERVER_KEY)
  }

  buildFCMAdmin(data, device_token, FCM_SERVER_KEY) {
    let payload = {
      notification: {
        notify_type: data.notify_type,
        title: data.title,
        body: data.text
      },
      data: {
        notify_type: data.notify_type,
        title: data.title,
        body: data.text
      },
      registration_ids: [...device_token]
    }
    console.log("=======Payload=======ADMIN", payload)
    this.sendFCM(payload, FCM_SERVER_KEY)
  }

  sendFCM(payload, FCM_SERVER_KEY) {
    fcm = new FCM(FCM_SERVER_KEY)
    fcm.send(payload, (err, response) => {
      if (err) {
        console.log('====================fcm error ::', err)
      } else {
        console.log('================Notification Sent')
        console.log(response)
      }
    })
  }

  async readNotification(body, notification_table) {
    let data = {
      is_read: 1,
      modified_date: dateHelper.getCurrentTimeStamp()
    }
    if (body.type == 1) {  // single notification read (from push notification)
      return await db.update(notification_table, `unique_id = '${body.unique_id}'`, data)
    } else {  // all notifications read (from notification section)
      return await db.update(notification_table, `receiver_id = ${body.user_id}`, data);
    }
  }

  async getNotifications(body, notification_table, notification_id) {
    let condition = ` receiver_id = ${body.user_id} AND is_deleted = 0 ORDER BY ${notification_id} DESC`
    let pagination = '', selectParams = '*'
    pagination = ` LIMIT ${Number(config.notificaionCount)} OFFSET ${Number(config.notificaionCount) * (Number(body.page_no) - 1)}`
    let notifications = await db.select(notification_table, selectParams, condition + pagination)
    condition = ` receiver_id = ${body.user_id}`
    let notificationCount = await db.select(notification_table, `COUNT(*)`, condition)
    return { notifications, notificationCount: notificationCount[0].count }
  }

  async deleteNotification(body, notification_table, notification_id) {
    let data = {
      is_deleted: 1,
      modified_date: dateHelper.getCurrentTimeStamp()
    },
      condition = ` ${notification_id} = ${body.notification_id}`
    return await db.update(notification_table, condition, data)
  }

  async getUnreadCounts(user_id) {
    let query = ` receiver_id = ${user_id} AND is_read = 0`
    let selectParams = `COUNT(notification_id) as unread_count`
    return await db.select('lb_notifications', selectParams, query)
  }
}

module.exports = new NotificationHelper()
