const promise = require('bluebird')
const dateHelper = require('../../utils/dateHelper')
const db = require('../../utils/db')
const config = require('../../utils/config')
const VideoHelper = require('./VideoHelper')
const PaytmChecksum = require('../../utils/PaytmChecksum')


class CommonHelper {

    async addRemoveFavorites(body) {
        try {

            let selectParams = '*';
            let whereFavorites = `user_id= ${body.user_id} AND recipe_id=${body.recipe_id} `;

            let recipe_favorites = await db.getById('favorites', selectParams, whereFavorites);

            let requestData = {
                user_id: body.user_id,
                recipe_id: body.recipe_id
            };

            if (recipe_favorites.id) {
                let where = `id=${recipe_favorites.id}`;
                await db.delete('favorites', where)
            } else {
                await db.insert('favorites', requestData)
            }

            let data = {};
            return { data };
        } catch (error) {
            return promise.reject(error)
        }
    }

    async addRating(body) {
        try {

            let selectParams = '*';
            let whereRating = `user_id= ${body.user_id} AND recipe_id=${body.recipe_id} `;
            let recipe_ratings = await db.getById('rating', selectParams, whereRating);

            let requestData = {
                user_id: body.user_id,
                recipe_id: body.recipe_id,
                ratings: body.ratings
            };

            if (recipe_ratings.id) {
                let where = `id=${recipe_ratings.id}`;
                await db.update('rating', where, requestData)
            } else {
                await db.insert('rating', requestData)
            }

            let data = {};
            return { data };
        } catch (error) {
            return promise.reject(error)
        }
    }

    async addComment(body) {
        try {
            let requestData = {
                user_id: body.user_id,
                recipe_id: body.recipe_id,
                name: body.name,
            };

            await db.insert('comment', requestData)

            let data = {};
            return { data };
        } catch (error) {
            return promise.reject(error)
        }
    }

    async getPaymentOption(body) {
        try {
            let selectParams = '*';
            let result = await db.select('payment_option', selectParams, '');

            let data = {};
            await result.forEach(function (row) {
                if (row.name == 'PayTm') {
                    data['paytm'] = row;
                }
                if (row.name == 'FlutterWave') {
                    data['flutterwave'] = row;
                }
                if (row.name == 'Razorpay') {
                    data['razorpay'] = row;
                }
                if (row.name == 'Paypal') {
                    data['paypal'] = row;
                }
                if (row.name == 'InApp Purchage') {
                    data['inapppurchage'] = row;
                }
            });

            return { data };
        } catch (error) {
            return promise.reject(error)
        }
    }

    async getPaymentToken(body) {
        var paytmParams = {};

        /* Generate Checksum via Array */

        /* initialize an array */
        const order_id = body.order_id ? body.order_id : 1;

        let selectParams = '*';
        let where = 'name="PayTm"'
        let payment_option = await db.getById('payment_option', selectParams, where);
        let MERCHANT_ID = '';
        let MERCHANT_KEY = '';
        if (payment_option.id) {
            if (payment_option.is_live == 1) {
                MERCHANT_ID = payment_option.live_key_1;
                MERCHANT_KEY = payment_option.live_key_2;
            } else {
                MERCHANT_ID = payment_option.test_key_1;
                MERCHANT_KEY = payment_option.test_key_2;
            }
        }

        paytmParams["MID"] = MERCHANT_ID;
        paytmParams["ORDERID"] = order_id;
        paytmParams["CUST_ID"] = body.CUST_ID ? body.CUST_ID : '';
        paytmParams["CHANNEL_ID"] = body.CHANNEL_ID ? body.CHANNEL_ID : '';
        paytmParams["TXN_AMOUNT"] = body.TXN_AMOUNT ? body.TXN_AMOUNT : '';
        paytmParams["WEBSITE"] = body.WEBSITE ? body.WEBSITE : '';
        paytmParams["CALLBACK_URL"] = body.CALLBACK_URL ? body.CALLBACK_URL : '';
        paytmParams["INDUSTRY_TYPE_ID"] = body.INDUSTRY_TYPE_ID ? body.INDUSTRY_TYPE_ID : '';

        /**
        * Generate checksum by parameters we have
        * Find your Merchant Key in your Paytm Dashboard at https://dashboard.paytm.com/next/apikeys 
        */
        let datas = {}, paytmCheck = [], verifySignature = [];
        var paytmChecksum = PaytmChecksum.generateSignature(paytmParams, MERCHANT_KEY);
        await paytmChecksum.then(function (result) {

            paytmCheck.push(result);
            var verifyChecksum = PaytmChecksum.verifySignature(paytmParams, MERCHANT_KEY, result);
            verifySignature.push(verifyChecksum);

        }).catch(function (error) {
            return promise.reject(error)
        });

        datas.paytmChecksum = paytmCheck[0];
        datas.verifySignature = verifySignature[0];
        let data = datas;
        return { data };
    }
}

module.exports = new CommonHelper()