const db = require('../../utils/db')
const promise = require('bluebird')
const joi = require('joi')
const joiValidator = require('../../utils/joiValidator')

/**
 * This CommonValidator class contains car brand model and type add edit list related API's validation.
 */

class RecipeValidator {

    async addRecipeValidator(body) {
        try {
            let schema = joi.object().keys({
                chef_id: joi.number().integer().required(),
                category_id: joi.number().integer().required(),
                name: joi.string().required(),
                cooking_time: joi.string().required(),
                food_type: joi.string().required(),
                recipe_type: joi.string().required(),
                image: joi.string().optional(),
                calorie: joi.string().required(),
                ingredients: joi.string().required(),
                steps: joi.string().required(),
                video: joi.string().required(),
            })
            await joiValidator.validateJoiSchema(body, schema);
        } catch (error) {
            return promise.reject(error)
        }
    }

    async addMealValidator(body) {
        try {
            let schema = joi.object().keys({
                recipe_id: joi.number().integer().required(),
                user_id: joi.number().integer().required(),
                date: joi.string().required(),
                meal_type: joi.string().required()
            })
            await joiValidator.validateJoiSchema(body, schema);
        } catch (error) {
            return promise.reject(error)
        }
    }

    async getMealValidator(body) {
        try {
            let schema = joi.object().keys({
                user_id: joi.number().integer().required()
            })
            await joiValidator.validateJoiSchema(body, schema);
        } catch (error) {
            return promise.reject(error)
        }
    }
}

module.exports = new RecipeValidator()