const db = require('../../utils/db')
const promise = require('bluebird')
const joi = require('joi')
const joiValidator = require('../../utils/joiValidator')

/**
 * This userValidator class contains car brand model and type add edit list related API's validation.
 */


class UserValidator {
    async getUserProfileValidator(body) {
        try {
            let schema = joi.object().keys({
                id: joi.number().integer().required()
            })
            await joiValidator.validateJoiSchema(body, schema);
        } catch (error) {
            return promise.reject(error)
        }
    }

    async updateProfileValidator(body) {
        try {
            let schema = joi.object().keys({
                id: joi.number().integer().required(),
                name: joi.string().optional(),
                mobile: joi.string().optional(),
                email: joi.string().optional()
            })
            await joiValidator.validateJoiSchema(body, schema);
        } catch (error) {
            return promise.reject(error)
        }
    }


    async becomeChefValidator(body) {
        try {
            let schema = joi.object().keys({
                user_id: joi.number().integer().required()
            })
            await joiValidator.validateJoiSchema(body, schema);
        } catch (error) {
            return promise.reject(error)
        }
    }

    async registerValidator(body) {
        try {
            let schema = joi.object().keys({
                name: joi.string().required(),
                mobile: joi.string().required(),
                email: joi.string().required(),
                password: joi.string().required(),
            })
            await joiValidator.validateJoiSchema(body, schema);
        } catch (error) {
            return promise.reject(error)
        }
    }

    async loginValidator(body) {
        try {
            let schema = joi.object().keys({
                type: joi.string().required(),
                name: joi.string().optional(),
                mobile: joi.string().optional(),
                email: joi.string().optional(),
                password: joi.string().optional().allow('')
            })
            await joiValidator.validateJoiSchema(body, schema);
        } catch (error) {
            return promise.reject(error)
        }
    }
    async imageUploadValidator(body) {
        try {
            let schema = joi.object().keys({
                id: joi.number().required(),
                image: joi.string().optional()
            })
            await joiValidator.validateJoiSchema(body, schema);
        } catch (error) {
            return promise.reject(error)
        }
    }
}

module.exports = new UserValidator()