const express = require('express')
const app = express()
const logger = require('morgan')
const cors = require('cors')
const bodyParser = require('body-parser')

const config = require('./api/utils/config')
const db = require('./api/utils/db')
const v1Routes = require('./routes/V1routes')

logger.token('date', () => {
  return new Date().toLocaleString("en-US", { timeZone: "America/New_York" })
})
app.use(logger('[:date[]] :remote-addr ":method :url HTTP/:http-version" :status '))
app.use(bodyParser.urlencoded({ extended: true, limit: '200mb' }))
app.use(bodyParser.json({ limit: '200mb' }))
app.use(express.json())
app.use(express.urlencoded({ extended: true }))
app.use(cors())

app.use((req, res, next) => {
  res.setHeader('Access-Control-Allow-Origin', '*')
  res.setHeader('Access-Control-Allow-Methods', 'GET, POST, OPTIONS, PUT, PATCH, DELETE')
  res.setHeader('Access-Control-Allow-Headers', '*')
  res.setHeader('Access-Control-Allow-Credentials', true)
  next()
})

app.use('/assets', express.static('assets'))
app.use('/static', express.static('static'))

app.use('/api/v1', v1Routes)


db.getConnection()
  .then(() => {
    app.listen(config.port, async () => {
      console.log(`Server Started at http://${config.host}:${config.port}`)

    })
  })
  .catch((error) => {
    console.log(error)
  })

module.exports = app;